/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.wsapi.service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jbi.management.ComponentLifeCycleMBean;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.extension.wsapi.KernelWebService;
import org.ow2.petals.kernel.ws.api.JBIArtefactsService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.to.ServiceAssembly;
import org.ow2.petals.kernel.ws.api.to.SharedLib;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "webservice", signature = JBIArtefactsService.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class JBIArtefactsServiceImpl implements KernelWebService, JBIArtefactsService {
	
    private static final String COMPONENT_LOGGER_NAME = "Petals.Extensions.WsApi.JBIArtefactsService";

    private LoggingUtil log;

    @org.objectweb.fractal.fraclet.annotation.annotations.Service(name = "component")
    private Component component;

    @Requires(name = "adminService")
    private AdminService adminService;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(COMPONENT_LOGGER_NAME));
        this.log.debug("Starting...");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.debug("Stopping...");
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public List<ServiceAssembly> getServiceAssemblies() throws PEtALSWebServiceException {
        List<ServiceAssembly> result = new ArrayList<ServiceAssembly>();

        Map<String, ServiceAssemblyLifeCycle> sas = this.adminService.getServiceAssemblies();
        if (sas != null) {
            Iterator<String> iter = sas.keySet().iterator();
            while (iter.hasNext()) {
                String saName = iter.next();
                ServiceAssemblyLifeCycle sal = sas.get(saName);

                ServiceAssembly sa = new ServiceAssembly();
                sa.setState(sal.getCurrentState());
                sa.setName(saName);
                if (sal.getServiceAssembly().getIdentification() != null) {
                    sa
                            .setDescription(sal.getServiceAssembly().getIdentification()
                                    .getDescription());
                }
                result.add(sa);
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<org.ow2.petals.kernel.ws.api.to.Component> getComponents()
            throws PEtALSWebServiceException {
        List<org.ow2.petals.kernel.ws.api.to.Component> result = new ArrayList<org.ow2.petals.kernel.ws.api.to.Component>();

        Map<String, ComponentLifeCycleMBean> lc = this.adminService
                .getBindingComponentsLifeCycles();
        if (lc != null) {
            Iterator<String> iter = lc.keySet().iterator();
            while (iter.hasNext()) {
                String name = iter.next();
                ComponentLifeCycleMBean clc = lc.get(name);
                org.ow2.petals.kernel.ws.api.to.Component c = new org.ow2.petals.kernel.ws.api.to.Component();
                c.setState(clc.getCurrentState());
                c.setName(name);
                c.setType(BINDING);
                result.add(c);
            }
        }

        lc = this.adminService.getServiceEnginesLifeCycles();
        if (lc != null) {
            Iterator<String> iter = lc.keySet().iterator();
            while (iter.hasNext()) {
                String name = iter.next();
                ComponentLifeCycleMBean clc = lc.get(name);
                org.ow2.petals.kernel.ws.api.to.Component c = new org.ow2.petals.kernel.ws.api.to.Component();
                c.setState(clc.getCurrentState());
                c.setName(name);
                c.setType(ENGINE);
                result.add(c);
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<SharedLib> getSharedLibraries() throws PEtALSWebServiceException {
        List<SharedLib> result = new ArrayList<SharedLib>();

        Map<String, SharedLibraryLifeCycle> lc = this.adminService.getSharedLibraries();
        if (lc != null) {
            Iterator<String> iter = lc.keySet().iterator();
            while (iter.hasNext()) {
                String name = iter.next();
                SharedLibraryLifeCycle slc = lc.get(name);
                SharedLib sl = new SharedLib();
                sl.setState(slc.getState());
                sl.setName(name);
                result.add(sl);
            }
        }
        return result;
    }
}
